#!/usr/bin/python
# coding: UTF-8
#
# see documentation for more information
#
version = 1.2
 
import time
import os.path
import sys

# set HOME var (path where this file is in)
HOME = sys.path[0]

# add lib directory to sys.path
sys.path.insert(1, os.path.join(HOME, "lib"))
sys.path.insert(1, os.path.join(HOME, "modules"))

from pprint   import pprint
from optparse import OptionParser


# locking facility
from locking import Lock

import fileDB


try:
    from pysnmpManager import Manager
    from pysnmpManager import Timeout
    from pysnmpManager import SNMPError
except ImportError, e:
    print "got error: " + str(e)
    nagiosExit(3, "could not import pysnmpManager, pls. add it to PYTHONPATH")
    

# configure logging
from mlog import getLogger
    
APP_NAME = "voice"


ifType = 18

IsdnOffset = 16


# configure option parser
#
# usage output
parser = OptionParser(usage="%prog -H <host> -c <community> (--in | --out) (-I | -l | -a) [options]", version="%prog " + str(version))

# mode parameters (one of them required)
parser.add_option("-l", "--learn",
                  dest="learn",
                  default=False,
                  action="store_true",
                  help="start %s in learn mode" % APP_NAME)
parser.add_option("-I", "--interface",
                  dest="interface",
                  default=None,
                  action="store",
                  type="int",
                  help="returns the status a single interfaxe")   
parser.add_option("-a", "--all",
                  dest="all",
                  default=False,
                  action="store_true",
                  help="returns the whole status of all ports")

# in/out - coming calls (one of both required)
parser.add_option("-i", "--in",
                  dest="inCalls",
                  default=None,
                  action="store_true",
                  help="get status of incoming calls")
parser.add_option("-o", "--out",
                  dest="outCalls",
                  default=False,
                  action="store_true",
                  help="get status of outgoing calls")   


# other must have parameters (all required)
parser.add_option("-H", "--host",
                  dest="host",
                  default=None,
                  action="store",
                  help="set hostname or ip")




parser.add_option("-c", "--community",
                  dest="community",
                  default="public",
                  action="store",
                  help="set community")


# nagios parameters (config creation and warn/critical levels)
parser.add_option("-n", "--nagios-config",
                  dest="nagiosConfig",
                  default=False,
                  action="store_true",
                  help="automatic nagios config creator")

parser.add_option("-C", "--critical",
                  dest="critical",
                  default=40,
                  action="store",
                  help="set critical value for nagios")
parser.add_option("-W", "--warn",
                  dest="warn",
                  default=30,
                  action="store",
                  help="set warn value for nagios") 
   


# optional parameters
parser.add_option("-L", "--log-file",
                  dest="logFile",
                  default=None,
                  action="store",
                  help="set path of log file (default log/HOSTNAME.log)")

parser.add_option("", "--log-level",
                  dest="logLevel",
                  default="info",
                  action="store",
                  help="set level (notset|debug|info|warning|error) (default info)")

parser.add_option("", "--port-data-file",
                  dest="portDataFile",
                  default=None,
                  action="store",
                  help="destination of port data file (default status/HOSTNAME.portdata)")   

parser.add_option("", "--port-status-file",
                  dest="portStatusFile",
                  default=None,
                  action="store",
                  help="destination of port status file (default status/HOSTNAME.portstatus)")   

parser.add_option("", "--lock-file",
                  dest="lockFile",
                  default=None,
                  action="store",
                  help="destination of lock file (default lock/HOSTNAME.lock)")   

parser.add_option("-d", "--debug",
                  dest="debug",
                  default=False,
                  action="store_true",
                  help="start %s in debug mode (max output)" % APP_NAME)

parser.add_option("-t", "--timeout",
                  dest="timeout",
                  default=60,
                  action="store",
                  type="int",
                  help="in seconds; after reaching this timeout the procedure will fail and UNKNOWN will be returned to nagios")   

parser.add_option("-P", "--port-ttl",
                  dest="portTTL",
                  default=290,
                  action="store",
                  type="int",
                  help="time to live for volatile status data")   

parser.add_option("-p", "--port",
                  dest="port",
                  default=161,
                  action="store",
                  type="int",
                  help="TCP port of agent (default 161)")

(options, args) = parser.parse_args()

# defining some often used oids
OID_ifType                      = (1,3,6,1,2,1,2,2,1,3)
OID_ifDescr                     = (1,3,6,1,2,1,2,2,1,2)
OID_ifOperStatus                = (1,3,6,1,2,1,2,2,1,8)
OID_isdnSignalingIfIndex        = (1,3,6,1,2,1,10,20,1,3,2,1,2)
OID_isdnLapdOperStatus          = (1,3,6,1,2,1,10,20,1,3,4,1,2)
OID_isdnSigStatsInCalls         = (1,3,6,1,2,1,10,20,1,3,3,1,1)
OID_isdnSigStatsOutCalls        = (1,3,6,1,2,1,10,20,1,3,3,1,3)



class DataCollector:
    """
    Collects Data From SNMP Device.
    Checks the interface type of every avaliable interface. Writes the information in a status file.
    """
    def __init__(self, snmpManager):
        
        self.manager = snmpManager
        self.data = {}
        
    def check(self):
        """
        Check the ports of the agent to find out which ports are E1 interfaces
        """
        self.data["data"] = {}
        
        # {
        #  10001 : 1,
        #  10002 : 2, ...
        # }
        portIfMapping = {}
        
        # snmpResponse = {ifTypeOid : {interface1 : ifType(interface1),
        #                              ...
        #                             }
        #                }
        
        logger.info("send ifType mib-snmp request")
        snmpRequestStart = time.time()
        try:
            snmpResponse = self.manager.getMib((OID_ifType,))
        except Timeout:
            logger.error("timeout reached")
            nagiosExit(3, "Timeout")
        except SNMPError:
            logger.error("SNMP ERROR")
            logger.error("errors may be:")
            logger.error("  - wrong community")
            logger.error("  - hostname could not be resolved")
            logger.error("  - host doesn't response on snmp requests on given port")
            
            nagiosExit(3, "SNMP Error")
            
        logger.debug("lookup time: %s" % (time.time() - snmpRequestStart))  
        
        ifTypeData                  = snmpResponse[OID_ifType]
        
        # self.data = \
        # {'data': {1: [10001, 'FastEthernet0/1', 1,  32],
        #           2: [10002, 'FastEthernet0/2', 1,  95],
        #           3: [10003, 'FastEthernet0/3', 2, 158], ...
        
        ifTypeDataKeys = ifTypeData.keys()
        ifTypeDataKeys.sort()
        for interfaceOid in ifTypeDataKeys:
            # interfaceOid: (1, 3, 6, 1, 2, 1, 2, 2, 1, 3, 10001)
            # interfaceNr:  10001
            interfaceNr   = interfaceOid[-1]
            interfaceType = int(ifTypeData[interfaceOid])
            
            if interfaceType == ifType:
                self.data["data"][interfaceNr] = [interfaceNr,]
        
        
        # all mib containing oids which ends with the ifNr
        IsdnPortMibs = [OID_isdnSignalingIfIndex]
        
        i = 1
        requestedOids       = []
        for interfaceNr in self.data["data"]:
            dataset = self.data["data"][interfaceNr]
            requestedOids.append(OID_ifDescr + (interfaceNr,))
            requestedOids.append(OID_ifOperStatus + (interfaceNr,))
            # OID_isdnSignalingIfIndex starts counting by 1 (not interfaceNr) 
            requestedOids.append(OID_isdnSignalingIfIndex + (i,))
            portIfMapping[i] = interfaceNr
            i += 1
            
        logger.info("send interface data - snmp request")
        snmpRequestStart = time.time()
        try:
            snmpResponse = self.manager.getOid(requestedOids)
        except Timeout:
            logger.error("timeout reached - %ss" % (time.time() - snmpRequestStart))
            nagiosExit(3, "Timeout")
            
        logger.debug("lookup time: %s" % (time.time() - snmpRequestStart))  
        
        ifOperStatusList = snmpResponse[OID_ifOperStatus].keys()
        ifOperStatusList.sort()
        ifDescrList = snmpResponse[OID_ifDescr].keys()
        ifDescrList.sort()
        
        for ifOperStatusOid, ifDescrOid in zip(ifOperStatusList, ifDescrList):
            ifOperStatus = snmpResponse[OID_ifOperStatus][ifOperStatusOid]
            ifNr = ifOperStatusOid[-1]
            
            # remove unconfigured interfaces
            if ifOperStatus <> 1:
                del self.data["data"][ifNr]
                continue
            
            
            ifDescr = snmpResponse[OID_ifDescr][ifDescrOid]
            self.data["data"][ifNr].append(ifDescr)
            self.data["data"][ifNr].append(ifOperStatus)
            
        
        for mib in IsdnPortMibs:
            for oid in snmpResponse[mib]:
                value = snmpResponse[mib][oid] - IsdnOffset
                portNr = oid[-1]
                ifNr = portIfMapping[portNr]
                # only append data to interfaces which are not removed (only for interfaces with ifOperstats != 2)
                if self.data["data"].has_key(ifNr):
                    self.data["data"][ifNr].append(value)
                    # Append the ISDN-Port Nr. to dataset
                    self.data["data"][ifNr].append(portNr)
               
        self.data["time"] = time.time()
        self.writeStatus()
        
    def writeStatus(self):
        """
        Dump status to status file in status dir
        """
        logger.debug("store interface port status in %s" % portDataFileName)
        fileDB.dump(self.data, portDataFileName)

    def run(self):
        self.check()

class PortChecker:
    def __init__(self, manager):
        self.manager = manager
        
        self.status  = Status(portStatusFileName)
        self.lock    = Lock(lockFileName)

    def loadDataCollectorStatus(self):
        logger.debug("load data collector status file '%s'" % portDataFileName)
        collectorStatus = fileDB.load(portDataFileName)
        self.collectorData = collectorStatus["data"]
       
    def checkPorts(self):
        """
        Check single interface.
        If the ttl of the interface is not reached the status will be returned from file.
        If ttl is reached a full snmp lookup will be requested.
        """
        try:
            self.status.load()
        except IOError:
            # status file is not readable
            logger.warning("can't load status file %s" % portStatusFileName)
            return self.snmpLookup()
        
            
        timeDiff = self.status.getTimeDiff()
        
        if timeDiff > options.portTTL:
            # port ttl reached
            logger.info("ttl reached, send new snmp request")
            return self.snmpLookup()
        else:
            # port ttl not reached
            logger.debug("ttl not reached, returned status of file")
            return self.status
            
    def snmpLookup(self):
        """
        Requests all interfaces of type ifType and stores them in status file
        """
        if self.lock.isSet():
            logger.warning("lock is set, return status of file without consideration of ttl")
            if not self.status.isFile():
                logger.error("process locked and status file not readable")
                logger.error("kill existing voice processes (if they are running) and delete lock (%s) by hand" % lockFileName)
                nagiosExit(3, "process locked and status file not readable")
            return self.status
            
        # set lock
        logger.debug("set lock to %s" % self.lock.fileName)
        # IOError could not be raised because the lock file could not exist here
        self.lock.close()
        
        
        logger.info("lookup all ports")
        logger.debug("check interfaces of DataCollector")
        
        # self.collectorData = \
        #          {1: [10001, 'FastEthernet0/1', 1,  32],
        #           2: [10002, 'FastEthernet0/2', 1,  95],
        #           3: [10003, 'FastEthernet0/3', 2, 158], ...
        try:
            self.loadDataCollectorStatus()
        except IOError, e:
            logger.error("status file of data collector checker could not be found")
            logger.error("start %s in learn mode" % APP_NAME)
            logger.debug("open lock")
            self.lock.open()
            nagiosExit(3, "status file of data collector could not be found")

        isdnIfIdPortMapping = {}
        portInterfaceIdMapping = {}
        
        requestedOids       = []
        for interfaceId in self.collectorData.keys():
            dataset = self.collectorData[interfaceId]
            isdnIfId = dataset[3]
            portNr   = dataset[4]
            
            requestedOids.append(OID_isdnLapdOperStatus + (isdnIfId,))
            requestedOids.append(OID_isdnSigStatsInCalls + (portNr,))
            requestedOids.append(OID_isdnSigStatsOutCalls + (portNr,))
            isdnIfIdPortMapping[isdnIfId] = portNr
            portInterfaceIdMapping[portNr] = interfaceId
        
        isndIfIdSpecificMibs    = [OID_isdnLapdOperStatus]
        portSpecificMibs        = [OID_isdnSigStatsInCalls, OID_isdnSigStatsOutCalls]
        
        
        snmpRequestStart = time.time()
        # send request
        try:
            snmpResponse = self.manager.getOid(requestedOids)
        except Timeout:
            logger.error("timeout reached - %ss" % (time.time() - snmpRequestStart))
            self.lock.open()
            nagiosExit(3, "Timeout")
        logger.debug("lookup time: %s" % (time.time() - snmpRequestStart))
        
        
        # set isdnSigStatsInCallsData
        for oid in snmpResponse[OID_isdnSigStatsInCalls].keys():
            portNr = oid[-1]
            interfaceId = portInterfaceIdMapping[portNr]
            value  = snmpResponse[OID_isdnSigStatsInCalls][oid]
            self.status.setIsdnSigStatsInCalls(interfaceId, value)
        
        # set isdnSigStatsOutCallsData    
        for oid in snmpResponse[OID_isdnSigStatsOutCalls].keys():
            portNr = oid[-1]   
            interfaceId = portInterfaceIdMapping[portNr]
            value  = snmpResponse[OID_isdnSigStatsOutCalls][oid]
            self.status.setIsdnSigStatsOutCalls(interfaceId, value)
           
        # set isdnLapdOperStatus
        for oid in snmpResponse[OID_isdnLapdOperStatus].keys():
            isdnSignalingIfIndex    = oid[-1]
            portNr                  = isdnIfIdPortMapping[isdnSignalingIfIndex]
            interfaceId             = portInterfaceIdMapping[portNr]
            
            value  = snmpResponse[OID_isdnLapdOperStatus][oid]
            self.status.setIsdnLapdOperStatus(interfaceId, value)
        
        self.status.setTime(time.time())
        
        self.status.dump()

        
        # after status is written the lock can be released
        logger.debug("remove lock %s" % self.lock.fileName)
        self.lock.open()
        
        return self.status

class Status:
    """
    Status class stores status data of all (interesting) interfaces of a snmp agent.
    It  provides also functions for calculating interesting stats.
    """
    def __init__(self, fileName):
        self.fileName   = fileName
        
        self.content    = {"data" : {},}
        self.data       = self.content["data"]
        
    def load(self):
        """
        Loads the status from status file.
        If the file doesn't exist or is not readable IOError will be reised
        """
        logger.debug("load port status file '%s'" % portStatusFileName)
        self.content = fileDB.load(self.fileName)
        
        self.data           = self.content["data"]
        self.timeStamp      = self.content["time"]

        try:
            self.lastTimeStamp  = self.content["last-time"]
        except KeyError:
            self.content["last-time"] = self.content["time"]
            self.lastTimeStamp  = self.content["last-time"]
        
    def dump(self):
        """
        Dumps the content of the instance of this class to status file.
        """
        logger.debug("store port status in file %s" % portStatusFileName)
        fileDB.dump(self.content, self.fileName)
        
        
    def calcInCallsPerMinute(self, portNr):
        """
        Returns the calls per minute calculated by the values given by the snmp device between the last run
        and this run.
        """
        
        try:
            portData = self.data[portNr]
        except KeyError:
            # port is not configured in port data file
            msg = "interface %s is not configured in status data (is not an valid interface or ifOperStatus is inactive)" % portNr
            logger.error(msg)
            nagiosExit(3, msg)
            
        inCalls     = portData[1]
        
        # handle overflow. Type is 32bit int - 4294967296
        # (e.g) inCalls = [4394967280, 13]
        # callDiff = 4394967296 - 4394967280 + 13 = 29
        try:
            if inCalls[1] < inCalls[0]:
                callDiff = 4394967296 - inCalls[0] + inCalls[1]
            else:
                # no overflow
                callDiff    = inCalls[1] - inCalls[0]
        except IndexError:
            # only one var exists, because of voice could only run one time
            return 0
             
        # time diff in minutes   
        timeDiff     = abs(self.timeStamp - self.lastTimeStamp) / 60.0
        callsPerHour = callDiff / timeDiff
        
        return int(callsPerHour)

    def calcOutCallsPerMinute(self, portNr):
        """
        Returns the calls per minute calculated by the values given by the snmp device between the last run
        and this run.
        """
        portData = self.data[portNr]
        
        outCalls    = portData[2]
        
        # handle overflow. Type is 32bit int - 4294967296
        # (e.g) inCalls = [4394967280, 13]
        # callDiff = 4394967296 - 4394967280 + 13 = 29
        try:
            if outCalls[1] < outCalls[0]:
                callDiff = 4394967296 - outCalls[0] + outCalls[1]
            else:
                # no overflow
                callDiff    = outCalls[1] - outCalls[0]
        except IndexError:
            # only one var exists, because of voice could only run one time
            return 0
        
        # time diff in minutes
        timeDiff        = abs(self.timeStamp - self.lastTimeStamp) / 60.0
        callsPerHour    = callDiff / timeDiff
        
        return int(callsPerHour)
        
    def getTimeDiff(self):
        """
        Returns the time difference between this run (actual time) and the last run in seconds.
        """
        timeDiff = time.time() - self.timeStamp
        return timeDiff
         
    def getPortData(self, portNr):
        """
        Returns the dataset of one port.
        """
        return self.data[portNr]
    
    def setTime(self, newTimeStamp):
        """
        Rotates the time stamps in status data. Removes the oldest entry. 
        The time stamp of the last run is stored in vars:
            self.lastTimestamp
            self.content["last-time"]
            
        The time stamp of this run (parameter newTimeStamp) is stored in vars:
            self.timeStamp
            self.content["time"]
        """
        # rotate time stamps
        try:
            self.content["last-time"] = self.timeStamp
            self.lastTimeStamp        = self.timeStamp
        except AttributeError:
            # timeStamp is not known, because status file could not be loaded
            self.timeStamp = newTimeStamp
        
        self.content["time"]      = newTimeStamp
        self.timeStamp            = newTimeStamp        

    def setIsdnSigStatsInCalls(self, interfaceId, calls):
        """
        Rotate values of isdnSigStatsInCalls. Removes the oldest entry and add the new one.
        There can only be 2 values. The first was created by the last run and the second by this run.
        """
        if not self.data.has_key(interfaceId):
            self.data[interfaceId] = [None, None, None]
                    
        if not type(self.data[interfaceId][1]) == list:
            self.data[interfaceId][1] = []
            
        isdnSigStatsInCalls = self.data[interfaceId][1]
        isdnSigStatsInCalls.append(calls)
            
        if len(isdnSigStatsInCalls) > 2:
            del isdnSigStatsInCalls[0]
            
    def setIsdnSigStatsOutCalls(self, interfaceId, calls):
        """
        Rotate values of isdnSigStatsOutCalls. Removes the oldest entry and add the new one.
        There can only be 2 values. The first was created by the last run and the second by this run.
        """
        if not self.data.has_key(interfaceId):
            self.data[interfaceId] = [None, None, None]
                    
        if not type(self.data[interfaceId][2]) == list:
            self.data[interfaceId][2] = []
            
        isdnSigStatsOutCalls = self.data[interfaceId][2]
        isdnSigStatsOutCalls.append(calls)
            
        if len(isdnSigStatsOutCalls) > 2:
            del isdnSigStatsOutCalls[0]
        
    def setIsdnLapdOperStatus(self, interfaceId, value):
        """
        Set the setIsdnLapdOperStatus of interfaceId.
        The setIsdnLapdOperStatus will not be rotatet. Only the value of this run will be stored.
        """
        self.data[interfaceId][0] = value
        
    def isFile(self):
        """
        Returns the existence bool of the file
        """
        return os.path.isfile(self.fileName)
    
    def getOverAllStatus(self):
        """
        Returns the sum of inCalls, outCalls and the sum of them as a list (with 3 elements)
        """
        inCalls  = 0
        outCalls = 0

        isdnLapdOperStatus = True       
        for interfaceId in self.data.keys():
            inCalls     += self.calcInCallsPerMinute(interfaceId)
            outCalls    += self.calcOutCallsPerMinute(interfaceId)
            if not self.checkIsdnLapdOperStatus( interfaceId ) :
                isdnLapdOperStatus = False
            
        result = [inCalls, outCalls, inCalls + outCalls, isdnLapdOperStatus]
        return result
        
    def checkIsdnLapdOperStatus(self, interface):
        """
        Returns True when isdn interface is lv3 (up), returns False when isdnLapdOperStatus is not 3
        
        @type interface: int
        @rtype: bool
        """    
        return self.getIsdnLapdOperStatus( interface ) == 3

    def getIsdnLapdOperStatus(self, interface):
        """
        Returns the isdnLapdOperStatus (1=down, 2=L1, 3=L2)
        
        @type interface: int
        @rtype: int
        """
        isdnLapdOperStatus = self.data[interface][0]
    
        return isdnLapdOperStatus

    
def learn():
    """
    This function will be executed, when -l / --learn parameter is set
    """
    logger.info("start %s in learn mode" % APP_NAME)
    # check interface types and write them to status file
    dataCollector = DataCollector(manager)
    dataCollector.run()
    
    if os.path.isfile(portStatusFileName):
        os.remove(portStatusFileName)

def checkInterface(interfaceId):
    """
    This function will be executed, when -I / --interface parameter is set
    """
    logger.info("start %s in interface(interface=%s) mode" % (APP_NAME, interfaceId))
    
    # check single port status
    # if port status is to old (ttl exceded) or status file is not reachable a snmp lookup is requested
    portChecker = PortChecker(manager)
        
    portStatus = portChecker.checkPorts()
    
    
    inCalls  = portStatus.calcInCallsPerMinute(interfaceId)
    outCalls = portStatus.calcOutCallsPerMinute(interfaceId)
    
    isdnLapdOperStatus = portChecker.status.getIsdnLapdOperStatus(interfaceId)
    
    # warn when isdnLapdOperStatus <= 2 (2:)
    # crit when isdnLapdOperStatus <= 1 (1:)
    perfData  = "isdnLapdOperStatus=%s;2:;1:;0;3 " % (isdnLapdOperStatus)
    perfData += "IN=%s;%s;%s;0;%s " % (inCalls, options.warn, options.critical, options.warn)
    perfData += "OUT=%s;%s;%s;0;%s " % (outCalls, options.warn, options.critical, options.warn)
    
    if not isdnLapdOperStatus == 3:
        # check failed
        msg = "interface %s has isdnLapdOperStatus %s but must have 3" % (interfaceId, isdnLapdOperStatus)
        logger.error(msg)
        nagiosExit(2, msg, perfData) 
    
    exitCode = None
    
    if options.inCalls:
        value = inCalls
    elif options.outCalls:
        value = outCalls
     
    if value < options.warn:
        exitCode = 0
    elif value < options.critical:
        exitCode = 1
    else:
        exitCode = 2
     
    nagiosExit(exitCode, str(value), perfData=perfData)
    
def checkOverAllStatus():
    """
    This function will be executed, when -a / --all parameter is set
    """
    portChecker     = PortChecker(manager)
    portStatus      = portChecker.checkPorts()
    overallStatus   = portStatus.getOverAllStatus()
    
    inCalls     = overallStatus[0]
    outCalls    = overallStatus[1]
    sum         = overallStatus[2]
    
    exitCode = None
    
    if options.inCalls:
        value = inCalls
        perfData  =  "SUM_IN=%s;%s;%s;0;%s " % (value, options.warn, options.critical, options.critical)
    elif options.outCalls:
        value = outCalls
        perfData  =  "SUM_OUT=%s;%s;%s;0;%s " % (value, options.warn, options.critical, options.critical)
    else:
        value = sum
        perfData  =  "SUM_ALL=%s;%s;%s;0;%s " % (value, options.warn, options.critical, options.critical)

    if not overallStatus[3]:
        # check failed
        msg = "One interface has isdnLapdOperStatus != 3" 
        logger.error(msg)
        nagiosExit(2, msg, perfData)
        
    if value < options.warn:
        exitCode = 0
    elif value < options.critical:
        exitCode = 1
    else:
        exitCode = 2
    
    nagiosExit(exitCode, value, perfData)
    
def createNagiosConfig():
    """
    """
    from nagiosConfigCreator    import NagiosConfigCreator
    from consoleTools           import ask
    
    nagiosConfigCreator = NagiosConfigCreator()
        
    while True:
        fileName = ask("Where do you wan't to store the created configuration file?", 
                       os.path.join(HOME, sessionName) + "_nagios.cfg", str)
        if os.path.isfile(fileName):
            check = ask("File already exists, do you want to replace it?", False, bool)
            if not check:
                continue
        break
    print
    
    # ask for hostname
    print "Hostname of CISCO SNMP-agent!"
    print "This hostname must be configured in Nagios"
    hostName = ask("What's the hostname?", options.host)
    print
    
    nagiosConfigCreator = NagiosConfigCreator()
    nagiosConfigCreator.host         = options.host
    nagiosConfigCreator.port         = options.port
    nagiosConfigCreator.community    = options.community
    nagiosConfigCreator.timeout      = options.timeout
    nagiosConfigCreator.portTTL      = options.portTTL
    
    nagiosConfigCreator.logFile        = logFileName
    nagiosConfigCreator.portDataFile   = portDataFileName
    nagiosConfigCreator.portStatusFile = portStatusFileName
    nagiosConfigCreator.lockFileName   = options.lockFile
    nagiosConfigCreator.logLevel       = options.logLevel
    
    nagiosConfigCreator.prepare()
    
    # ask for warn level
    portInWarn      = ask("Warn-level     for single ports (Incoming Calls)?", 30, int)
    portInCritical  = ask("Critical-level for single ports (Incoming Calls)?", 40, int)
    print
    portOutWarn     = ask("Warn-level     for single ports (Outgoing Calls)?", 30, int)
    portOutCritical = ask("Critical-level for single ports (Outgoing Calls)?", 40, int)
    print
    allInWarn       = ask("Warn-level     for all ports (Incoming Calls)?", portInWarn      * 10, int)
    allInCritical   = ask("Critical-level for all ports (Incoming Calls)?", portInCritical  * 10, int)
    print
    allOutWarn      = ask("Warn-level     for all ports (Outgoing Calls)?", portOutWarn     * 10, int)
    allOutCritical  = ask("Critical-level for all ports (Outgoing Calls)?", portOutCritical * 10, int)
    print
    allWarn         = ask("Warn-level     for all ports (Outgoing + Incoming Calls)?", allOutWarn       + allInWarn,        int)
    allCritical     = ask("Critical-level for all ports (Outgoing + Incoming Calls)?", allOutCritical   + allInCritical,    int)
    
    
    nagiosConfigCreator.portInWarn          = portInWarn
    nagiosConfigCreator.portInCritical      = portInCritical
    nagiosConfigCreator.portOutWarn         = portOutWarn
    nagiosConfigCreator.portOutCritical     = portOutCritical
    nagiosConfigCreator.allInWarn           = allInWarn
    nagiosConfigCreator.allInCritical       = allInCritical
    nagiosConfigCreator.allOutWarn          = allOutWarn
    nagiosConfigCreator.allOutCritical      = allOutCritical
    nagiosConfigCreator.allWarn             = allWarn
    nagiosConfigCreator.allCritical         = allCritical
    
    nagiosConfig =  "# auto generated nagios configuration\n"
    nagiosConfig += "# generated by voice.py version %s\n"      % version
    nagiosConfig += "# at %s\n"                                 % time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
    
    try:
        collectorStatus = fileDB.load(portDataFileName)
    except IOError:
        print "port status file does not exist or is not readable. Run voice in learn mode first."
        nagiosExit(3, "partdata-file does not exist")
        
    # add command definitions to nagiosConfig
    nagiosConfig += nagiosConfigCreator.getCheckAllCommand()
    
    nagiosConfig += nagiosConfigCreator.getCheckAllInCommand()
    nagiosConfig += nagiosConfigCreator.getCheckAllOutCommand()
    
    nagiosConfig += nagiosConfigCreator.getCheckPortInCommand()
    nagiosConfig += nagiosConfigCreator.getCheckPortOutCommand()
    
    nagiosConfig += "\n\n\n"
    nagiosConfig += nagiosConfigCreator.getCheckAllService()
    nagiosConfig += nagiosConfigCreator.getCheckAllInService()
    nagiosConfig += nagiosConfigCreator.getCheckAllOutService()
    nagiosConfig += "\n\n"
    
    for portNr in collectorStatus["data"].keys():
        nagiosConfig += nagiosConfigCreator.getCheckPortInService(portNr)
        nagiosConfig += nagiosConfigCreator.getCheckPortOutService(portNr)
        
        
    file = open(fileName, "w")
    file.write(nagiosConfig)
    file.close()
    
    print "nagios config creation finished"

        
def nagiosExit(exitCode, msg=None, perfData=None):
    """
    Prints message msg to stdout for nagios. Exits with exit code exitCode.
    """
    printMsg = ""
    
    nagiosMessages = {0 : "OK",
                      1 : "WARNING",
                      2 : "CRITICAL",
                      3 : "UNKNOWN/UNREACHABLE"}
   
    printMsg += nagiosMessages[exitCode] 
        
    if msg:
        printMsg += " - " + str(msg)
        
    if perfData:
        printMsg += " | " + perfData

    print printMsg
    
    sys.exit(exitCode)

def main():
    global sessionName
    global portDataFileName
    global portStatusFileName
    global lockFileName
    global logFileName
    global logger
    global manager
        
        
        
    if not options.host:
        print "No host given. Use -H parameter!"
        parser.print_help() 
        nagiosExit(3, "parameter error")
    if not options.port:
        print "No tcp-port given. Use -p parameter!"
        parser.print_help() 
        nagiosExit(3, "parameter error")    
    if not options.community:
        print "No community given. Use -c parameter!"
        parser.print_help()
        nagiosExit(3, "parameter error") 
    if not options.portTTL:
        print "No port ttl given. Use -P parameter!"
        parser.print_help()
        nagiosExit(3, "parameter error")
    if not options.timeout:
        print "No timeout given. Use -t parameter!"
        parser.print_help()
        nagiosExit(3, "parameter error")
    if not options.learn and not options.interface and not options.all and not options.nagiosConfig:
        print "Mode parameter is missing. Use -I (interface) | -a (over-all) | -l (learn) | -n (nagios-config) parameter!"
        parser.print_help()
        nagiosExit(3, "parameter error")
    
    
    sessionName = options.host
    if not options.portDataFile:
        portDataFileName   = os.path.join(HOME, "status/%s.portdata"    % sessionName)
    else:
        portDataFileName   = options.portDataFile
    if not options.portStatusFile:
        portStatusFileName = os.path.join(HOME, "status/%s.portstatus"  % sessionName) 
    else:
        portStatusFileName = options.portStatusFile
    if not options.lockFile:
        lockFileName    = os.path.join(HOME, "lock/%s.lock"             % sessionName)
    else:
        lockFileName = options.lockFile
    if not options.logFile:
        logFileName         = os.path.join(HOME, "log/%s.log"           % sessionName)
    else:
        logFileName         = options.logFile
    
    if options.debug:
        isDebug = False
    else:
        isDebug = True
        
    logger = getLogger(logFileName, isDebug, level=options.logLevel)
    
    
    logger.info("%s started" % APP_NAME)
    
    if options.interface:
        if not options.inCalls and not options.outCalls:
            print "use either outCalls (--out) or inCalls (--in) parameter"
            parser.print_help()
            nagiosExit(3, "parameter is missing, see log file")
     
              
    logger.debug("port-data-file   is '%s'" % portDataFileName)
    logger.debug("port-status-file is '%s'" % portStatusFileName)
    logger.debug("lock file        is '%s'" % lockFileName)
    logger.debug("log file         is '%s'" % logFileName)
    
    logger.debug("host:      %s" % options.host)
    logger.debug("port:      %s" % options.port)
    logger.debug("interface: %s" % options.interface)
    logger.debug("community: %s" % (len(options.community) * "*"))
    logger.debug("timeout:   %s" % options.timeout)
    manager = Manager(options.host, options.port, options.community, timeout=options.timeout)
    
    if options.learn:
        learn()
        
    if options.interface:
        checkInterface(options.interface)
        
    if options.all:
        checkOverAllStatus()
        
    if options.nagiosConfig:
        createNagiosConfig()
        
    
    
    
if __name__ == "__main__":
    main()