from os.path import isfile

def ask(question, value=None, type=None):
    """
    Console based question to the user.
    The question will be printed.
    
    Value is the default. When a user doesn't answer the question (by pressing enter-key)
    the default value will be given back.
    
    You can specify a type. For example if you want to get an int-object, but the user
    enters a string he must repeat answering the question.
    
    If type is bool a "(yes/no)" will be added to the question. When value is given
    this will be returned, when the user doesn't type in exactly the opposit.
    (e.g.) >>> ask("Do you really want ...", False, bool)
           Do you really want ... (yes/no) [no] y             # this will return False
           False                                              # because the user didn't typed
                                                              # the word 'yes' exactly
    
    """
    if type == bool:
        boolValues = {True : "yes", False : "no"}
        question += " (yes/no)"
        if value is not None:
            question += " [%s] " % boolValues[value]
    else:  
        question += " [%s] " % value
    
    while True:
        answer = raw_input(question)
        
        if answer == "":
            answer = value
            break
        
        if type is bool:
            if answer == "yes":
                return True
            elif answer == "no":
                return False
            else:
                return value
        elif type == file:
            if not isfile(answer):
                print "file %s doesn't exist" % answer
                continue
        elif type is not None:
            try:
                answer = type(answer)
            except ValueError:
                continue
            
        break
    
    return answer