import logging
import logging.handlers

def getLogger(logFile, quiet=False, fileSize=10, backupCount=10, level="debug"):
    """
    Returns a configured logger with different settings:
    
    logFile     - path to log file (file will be created automaticly)
    quiet       - don't log to console (stdout) when this is True
    fileSize    - size of logFile in MB
    backupCount - number of logfiles which should be stored (old log files will be deleted automaticly)
    
    maximum used disk space = fileSize * backupCount [MB]
    """
    logger = logging.getLogger('')
    
    levels = {
              "notset"  : logging.NOTSET,
              "debug"   : logging.DEBUG,
              "info"    : logging.INFO,
              "warning" : logging.WARNING,
              "error"   : logging.ERROR,
              }
    
    try:
        logLevel = levels[level]
    except KeyError:
        logLevel = logging.DEBUG 
    
    if not quiet:
        # define a Handler which writes DEBUG messages or higher to the sys.stdout
        from sys import stdout
        consoleHandler = logging.StreamHandler(stdout)
        consoleHandler.setLevel(logging.DEBUG)
        # set a format which is simpler for console use
        consoleFormatter = logging.Formatter('%(asctime)s %(levelname)-8s %(message)s', "%Y-%m-%d %H:%M:%S")
        # tell the handler to use this format
        consoleHandler.setFormatter(consoleFormatter)
        # add the handler to the root logger
        logger.addHandler(consoleHandler)
    
    rotatingFileHandler = logging.handlers.RotatingFileHandler(logFile, maxBytes=1024*1024*fileSize, backupCount=backupCount)
    rotatingFileFormatter = logging.Formatter('%(asctime)s %(levelname)-8s %(message)s', "%Y-%m-%d %H:%M:%S")
    rotatingFileHandler.setFormatter(rotatingFileFormatter)
    logger.addHandler(rotatingFileHandler)
    logger.setLevel(logLevel)
    return logger

if __name__ == "__main__":
    logger = getLogger("test.log")
    logger.debug("hallo")